SuperStrict

Global GWidth :Int = 800
Global GHeight:Int = 600

Const KIND_PROTON  :Int = 0
Const KIND_ELECTRON:Int = 1
Const KIND_NEUTRON :Int = 2

Const STATE_MENU:Int	= 1
Const STATE_AI:Int	= 2
Const STATE_GAME:Int	= 3

Global Colors:Int[][] = [ [ 128, 128, 128 ], [ 255, 255, 255 ], [ 255, 0, 0 ], [ 255, 255, 0 ], [ 0, 0, 255 ], [ 0, 255, 0 ] ]
Global DeltaTimer:Int = MilliSecs(), DeltaTime:Float, AlphaTimer:Int

Global HeaderImage:TImage, MenuImages:TImage, GameOverImage:TImage, YouWinImage:TImage, LevelImage:TImage, FatFont:TImage

Global GameState:Int = STATE_MENU, CurrLevel:Int, Levels:TList[]

Init()

Local Timer:TTimer = CreateTimer( 60 )

While Not AppTerminate()
	Cls
	
	Select GameState
		Case STATE_MENU
			DrawImage HeaderImage, ( GWidth - HeaderImage.Width )/2, 50
			
			SetColor 255, 255, 0
			
			For Local I:Int = 0 To 1
				If MouseX() > GWidth/2 - 150 And MouseX() < GWidth/2 + 150 And ..
				   MouseY() > 280 + I*100 And MouseY() < 320 + I*100 Then
					SetBlend LIGHTBLEND
					DrawImage MenuImages, GWidth/2, 300 + I*100, I*2 + 1
					SetBlend ALPHABLEND
					
					If MouseHit( MOUSE_LEFT )
						If I Then
							End
						Else
							GameState = STATE_AI
						EndIf
					EndIf
				EndIf
				
				DrawImage MenuImages, GWidth/2, 300 + I*100, I*2
			Next
			
			SetColor 255, 255, 255
			
			If KeyHit( KEY_ESCAPE ) Then End
		Case STATE_AI
			SetColor 255, 255, 0
			
			For Local I:Int = 0 To 3
				If I And MouseX() > GWidth/2 - 150 And MouseX() < GWidth/2 + 150 And ..
				   MouseY() > 130 + I*100 And MouseY() < 170 + I*100 Then
					SetBlend LIGHTBLEND
					DrawImage MenuImages, GWidth/2, 150 + I*100, I*2 + 5
					SetBlend ALPHABLEND
					
					If MouseHit( MOUSE_LEFT )
						TAI.Difficulty = I - 1
						
						InitGame()
						
						GameState = STATE_GAME
					EndIf
				EndIf
				
				DrawImage MenuImages, GWidth/2, 150 + I*100, I*2 + 4
			Next
			
			SetColor 255, 255, 255
			
			If KeyHit( KEY_ESCAPE ) Then GameState = STATE_MENU
		Case STATE_GAME
			DeltaTime = ( MilliSecs() - DeltaTimer )*0.062
			DeltaTimer = MilliSecs()
			
			TAtom.Input()
			
			TAtom.Update()
			TAttackAtom.Update()
			
			TAI.UpdateAll()
			
			TAtom.Render()
			TAttackAtom.Render()
			
			SetColor 255, 255, 0
			SetAlpha Max( Min( 1.5 - ( MilliSecs() - AlphaTimer )*0.001, 1.0 ), 0.0 )
			
			DrawImage LevelImage, GWidth/2 - 70, GHeight/2
			
			SetScale 0.85, 0.85
			If CurrLevel < 9 Then
				DrawImage FatFont, GWidth/2 + 75, GHeight/2 - 6, CurrLevel + 1
			Else
				DrawImage FatFont, GWidth/2 +  75, GHeight/2 - 6, ( CurrLevel + 1 )/10
				DrawImage FatFont, GWidth/2 + 115, GHeight/2 - 6, ( CurrLevel + 1 ) Mod 10
			EndIf
			
			SetScale 1.0, 1.0
			SetAlpha 1.0
			SetColor 255, 255, 255
			
			If KeyHit( KEY_ESCAPE ) Then GameState = STATE_MENU
	End Select
	
	Flip 0
	WaitTimer Timer
Wend
End

Function Init()
	Graphics GWidth, GHeight
	
	HeaderImage = LoadImage( "Gfx\Header.png" )
	
	TAtom.Frames = LoadAnimImage( "Gfx\Atoms.png", 64, 64, 0, 100 )
	SetImageHandle TAtom.Frames, 32, 32
	
	AutoMidHandle True
	
	MenuImages = LoadAnimImage( "Gfx\Menue.png", 300, 40, 0, 12 )
	GameOverImage = LoadImage( "Gfx\GameOver.png" )
	YouWinImage = LoadImage( "Gfx\YouWin.png" )
	
	AutoMidHandle False
	
	LevelImage = LoadImage( "Gfx\Level.png" )
	FatFont = LoadAnimImage( "Gfx\FontFatBig.png", 40, 60, 0, 10 )
	
	TAtom.Font   = LoadAnimImage(  "Gfx\FontFat.png", 10, 15, 0, 10  )
	
	TAtom.Image = LoadImage( "Gfx\Atom.png" )
	TAtom.SelectionImg = LoadImage( "Gfx\Ring.png" )
	TAtom.TextRing = LoadImage( "Gfx\TextRing.png" )
	TAtom.TextRingNarrow = LoadImage( "Gfx\TextRingNarrow.png" )
	
	TAtom.LineImg = LoadImage( "Gfx\Line.png" )
	SetImageHandle TAtom.LineImg, 0, 8
	
	TAtom.ElectronImage = LoadImage( "Gfx\Electron.png" )
	SetImageHandle TAtom.ElectronImage, 16, 16
	
	SetBlend ALPHABLEND
End Function

Function LoadNextLevel()
	CurrLevel :+ 1
	
	TAtom.Atoms = Levels[ CurrLevel ]
	
	TAttackAtom.Attacks.Clear()
	
	TAI.BuildUndirectedGraph()
	
	AlphaTimer = MilliSecs()
End Function

Function InitGame()
	TAtom.Atoms.Clear()
	TAttackAtom.Attacks.Clear()
	TAI.Graph = Null
	TAI.AIs.Clear()
	
	For Local I:Int = 2 To 5
		Local AI:TAI = New TAI
		AI.Color = I
	Next
	
	DeltaTimer = MilliSecs()
	
	CurrLevel = -1
	
	Levels = Null
	
	
	Local Instream:TStream = ReadFile( "Levels.lvl" )
	
	While Not Instream.Eof()
		Local Count:Int = Instream.ReadInt()
		
		For Local I:Int = 0 Until Count
			Local X:Int = Instream.ReadShort()
			Local Y:Int = Instream.ReadShort()
			Local Kind:Int = Instream.ReadShort()
			Local Color:Int = Instream.ReadShort()
			
			TAtom.Create( Color, X, Y, Kind )
		Next
		
		Levels :+ [ TAtom.Atoms ]
		
		TAtom.Atoms = New TList
	Wend
	
	Levels = Levels[ .. Levels.Length ]
	
	LoadNextLevel()
	
	TAI.BuildUndirectedGraph()
End Function

Function RotateAroundAxis( VectorX:Float, VectorY:Float, VectorZ:Float, AxisX:Float, AxisY:Float, AxisZ:Float, Angle:Float, RotX:Float Var, RotY:Float Var, RotZ:Float Var )
	Local C:Float = Cos( Angle )
	Local S:Float = Sin( Angle )
	
	RotX = VectorX*( AxisX*AxisX + ( 1 - AxisX*AxisX )*C ) + VectorY*( AxisX*AxisY*( 1 - C ) - AxisZ*S )     + VectorZ*( AxisX*AxisZ*( 1 - C ) + AxisY*S     )
	RotY = VectorX*( AxisX*AxisY*( 1 - C ) + AxisZ*S     ) + VectorY*( AxisY*AxisY + ( 1 - AxisY*AxisY )*C ) + VectorZ*( AxisY*AxisZ*( 1 - C ) - AxisX*S     )
	RotZ = VectorX*( AxisX*AxisZ*( 1 - C ) - AxisY*S     ) + VectorY*( AxisY*AxisZ*( 1 - C ) + AxisX*S     ) + VectorZ*( AxisZ*AxisZ + ( 1 - AxisZ*AxisZ )*C )
End Function

Type TAttackGroup
	Field Task:Int
	
	Field Color:Int
	
	Field Target:TAtom
	
	Field Size:Int
	
	Field Charge:Int[ 3 ]
	
	Field LastTime:Float
	Field LastAttack:TAttackAtom
	
	Method New()
		LastTime = 0.0
	End Method
End Type

Type TAI
	Const DIFFICULTY_EASY:Int	= 0
	Const DIFFICULTY_NORMAL:Int	= 1
	Const DIFFICULTY_HARD:Int	= 2
	
	Const TASK_ATTACK :Int = 1
	Const TASK_DEFEND :Int = 2
	Const TASK_CONQUER:Int = 3
	
	Global DifficultyDelays:Int[] = [ 6000, 4000, 3000 ]
	
	Global AIs:TList = New TList
	
	Global Difficulty:Int = DIFFICULTY_NORMAL
	
	Global Graph:TList[]
	
	Field Color:Int
	
	Field LastUpdate:Int
	
	Field TaskCounter:Int
	Field RunningTasks:TAttackGroup[ 256 ]
	
	Method New()
		AIs.AddLast( Self )
		
		LastUpdate = MilliSecs() - 300000
	End Method
	
	Method AddTask( Kind:Int, Group:TAttackGroup )
		RunningTasks[ TaskCounter ] = Group
		
		Group.Task = Kind
		
		TaskCounter :+ 1
	End Method
	
	Method FindTask:TAttackGroup( Kind:Int, Target:TAtom )
		For Local I:Int = 0 Until TaskCounter
			If RunningTasks[ I ].Task = Kind And RunningTasks[ I ].Target = Target Then Return RunningTasks[ I ]
		Next
	End Method
	
	Method Update()
		If MilliSecs() - LastUpdate < DifficultyDelays[ Difficulty ] Then Return
		
		Local ConquerD:Int = 1000000, ConquerA:TAtom, ConquerI:Int
		Local  AttackD:Int = 1000000,  AttackA:TAtom,  AttackI:Int
		
		For Local I:Int = 0 Until TaskCounter
			If Not RunningTasks[ I ].LastAttack Then
				TaskCounter :- 1
				
				RunningTasks[ I ] = RunningTasks[ TaskCounter ]
				
				RunningTasks[ TaskCounter ] = Null
				
				Exit
			EndIf
		Next
		
		Local Index:Int
		For Local Atom:TAtom = EachIn TAtom.Atoms
			If Not Atom.Color Then
				If Not FindTask( TASK_CONQUER, Atom ) Then
					For Local Neighbour:TAtom = EachIn Graph[ Index ]
						If Neighbour.Color = Color And Neighbour.Charge[ 0 ] + Neighbour.Charge[ 1 ] + Neighbour.Charge[ 2 ] >= 10 Then
							Local DX:Int = Atom.X - Neighbour.X
							Local DY:Int = Atom.Y - Neighbour.Y
							
							Local D:Int = DX*DX + DY*DY
							
							If D < ConquerD Then
								ConquerD = D
								ConquerA = Atom
								ConquerI = Index
							EndIf
							
							Exit
						EndIf
					Next
				EndIf
			ElseIf Atom.Color <> Color And Not FindTask( TASK_ATTACK, Atom ) Then
				Local Defense:Float = Atom.Charge[ 0 ]*TAtom.HP[ 0 ] + Atom.Charge[ 1 ]*TAtom.HP[ 1 ] + Atom.Charge[ 2 ]*TAtom.HP[ 2 ]
				
				Local Attack:Float = 0.0
				For Local Neighbour:TAtom = EachIn Graph[ Index ]
					If Neighbour.Color = Color Then
						Attack :+ Neighbour.Charge[ 0 ]/2*TAtom.Attack[ 0 ] + Neighbour.Charge[ 1 ]/2*TAtom.Attack[ 1 ] + Neighbour.Charge[ 2 ]/2*TAtom.Attack[ 2 ]
						
						Local DX:Int = Atom.X - Neighbour.X
						Local DY:Int = Atom.Y - Neighbour.Y
						
						Local D:Int = DX*DX + DY*DY
						
						Local SlowestSpeed:Float
						For Local I:Int = 0 To 2
							If Neighbour.Charge[ I ] Then SlowestSpeed = TAttackAtom.Speed[ I ]
						Next
						
						Local ProducedDefense:Float = TAtom.HP[ Atom.Kind ]*( Min( TAtom.Production[ Atom.Kind ]*Sqr( D )/SlowestSpeed + Atom.CurrElectron + Atom.Charge[ Atom.Kind ], 100 ) - Atom.Charge[ Atom.Kind ] )
						
						If Attack > Defense + ProducedDefense Then
							Local Group:TAttackGroup = New TAttackGroup
							
							For Local AttackAtom:TAtom = EachIn Graph[ Index ]
								If AttackAtom.Color = Color Then
									AttackAtom.SendElectrons( Atom, Group )
									
									If AttackAtom = Neighbour Then Exit
								EndIf
							Next
							
							AddTask( TASK_ATTACK, Group )
							
							LastUpdate = MilliSecs()
							
							Return
						EndIf
					EndIf
				Next
			EndIf
			
			Index :+ 1
		Next
		
		If ConquerA:TAtom Then
			For Local Neighbour:TAtom = EachIn Graph[ ConquerI ]
				If Neighbour.Color = Color And Neighbour.Charge[ 0 ] + Neighbour.Charge[ 1 ] + Neighbour.Charge[ 2 ] >= 10 Then
					Local Group:TAttackGroup = New TAttackGroup
					
					Neighbour.SendElectrons( ConquerA, Group )
					
					AddTask( TASK_CONQUER, Group )
					
					Exit
				EndIf
			Next
			
			LastUpdate = MilliSecs()
		EndIf
	End Method
	
	Method FindNearestAtom:TAtom( Source:TAtom, MinElectrons:Int )
		Local NearestAtom:TAtom, NearestD:Int = 1000000
		
		For Local Atom:TAtom = EachIn TAtom.Atoms
			If Atom.Color = Color And Atom.Charge[ 0 ] + Atom.Charge[ 1 ] + Atom.Charge[ 2 ] >= MinElectrons Then
				Local DX:Int = Atom.X - Source.X
				Local DY:Int = Atom.Y - Source.Y
				
				Local D:Int = DX*DX + DY*DY
				
				If D < NearestD Then
					NearestD = D
					NearestAtom = Atom
				EndIf
			EndIf
		Next
		
		Return NearestAtom
	End Method
	
	Function UpdateAll()
		For Local AI:TAI = EachIn AIs
			AI.Update()
		Next
	End Function
	
	Function BuildUndirectedGraph()
		Graph = New TList[ TAtom.Atoms.Count() ]
		
		Local Index:Int
		For Local Source:TAtom = EachIn TAtom.Atoms
			Graph[ Index ] = New TList
			
			For Local Atom:TAtom = EachIn TAtom.Atoms
				If Atom <> Source Then
					Local DX:Int = Atom.X - Source.X
					Local DY:Int = Atom.Y - Source.Y
					
					Atom.SortCriterion = DX*DX + DY*DY
					
					Graph[ Index ].AddLast( Atom )
				EndIf
			Next
			
			Graph[ Index ].Sort()
			
			Index :+ 1
		Next
	End Function
End Type

Type TAtom
	Global Production:Float[] = [ 0.05, 0.05, 0.05 ]
	Global HP:Float[] = [ 1.0, 2.0, 4.0 ]
	Global Attack:Float[] = [ 4.0, 2.0, 1.0 ]
	
	Global Image:TImage, ElectronImage:TImage, SelectionImg:TImage, LineImg:TImage, TextRing:TImage, TextRingNarrow:TImage, Font:TImage
	Global Frames:TImage
	
	Global Atoms:TList = New TList
	
	Global ParticleCount:Int[ 20 ]
	
	Field RingRotation:Float[ 2, 50 ]
	
	Field Angle:Float
	Field TurnSpeed:Float
	
	Field Color:Int
	Field Kind:Int
	
	Field Charge:Int[ 3 ]
	Field ChargeHealth:Float
	
	Field X:Int
	Field Y:Int
	
	Field Radius:Float
	
	Field CurrElectron:Float
	
	Field Hover:Int
	Field Selected:Int
	
	Field SortCriterion:Int
	
	Method New()
		Atoms.AddLast( Self )
		
		For Local I:Int = 0 Until 50
			RingRotation[ 0, I ] = Rnd( 0, 360 )
			RingRotation[ 1, I ] = Rnd( 0, 360 )
		Next
		
		TurnSpeed = Rnd( -1.0, 1.0 )
	End Method
	
	Method Compare:Int( Other:Object )
		Return SortCriterion - TAtom( Other ).SortCriterion
	End Method
	
	Method SendElectrons( Target:TAtom, Group:TAttackGroup = Null )
		For Local T:Int = 0 Until 3
			Charge[ T ] :/ 2
		Next
		
		TAttackAtom.Create( Self, Target, [ Charge[ 0 ], Charge[ 1 ], Charge[ 2 ] ], Group )
	End Method
	
	Method InsertAttack( AttackAtom:TAttackAtom )
		If Color = AttackAtom.Color Then
			For Local I:Int = 0 Until 3
				Charge[ I ] = Min( Charge[ I ] + AttackAtom.Charge[ I ], 99 )
				
				ChargeHealth = Max( ChargeHealth, HP[ I ]*Sgn( AttackAtom.Charge[ I ] ) )
			Next
		ElseIf Color Then
			Local ExitLoop:Int
			
			For Local I:Int = 0 To 2
				While AttackAtom.Charge[ I ]
					ChargeHealth :- Attack[ I ]
					
					Local MaxCharge:Int
					If Charge[ 2 ] Then MaxCharge = 2 ElseIf Charge[ 1 ] Then MaxCharge = 1
					
					While ChargeHealth < 0.0
						Charge[ MaxCharge ] :- 1
						ChargeHealth :+ HP[ MaxCharge ]
						
						If Charge[ MaxCharge ] <= 0 Then
							Charge[ MaxCharge ] = 0
							MaxCharge :- 1
							
							If MaxCharge = -1 Then
								Color = AttackAtom.Color
								
								For Local T:Int = 0 To 2
									Charge[ T ] = AttackAtom.Charge[ T ]
								Next
								
								Selected = False
								
								Return
							EndIf
						EndIf
					Wend
					
					AttackAtom.Charge[ I ] :- 1
				Wend
			Next
		Else
			Color = AttackAtom.Color
			
			ChargeHealth = Max( HP[ 0 ]*Sgn( AttackAtom.Charge[ 0 ] ), Max( HP[ 0 ]*Sgn( AttackAtom.Charge[ 1 ] ), HP[ 0 ]*Sgn( AttackAtom.Charge[ 2 ] ) ) )
			
			For Local I:Int = 0 To 2
				Charge[ I ] = AttackAtom.Charge[ I ]
			Next
		EndIf
	End Method
	
	Function Create:TAtom( Color:Int, X:Int, Y:Int, Kind:Int )
		Local Atom:TAtom = New TAtom
		
		Atom.Color = Color
		Atom.X = X
		Atom.Y = Y
		Atom.Radius = 29.0
		Atom.Kind = Kind
		If Atom.Color Then Atom.Charge[ Kind ] = 1
		
		Return Atom
	End Function
	
	Function Input()
		Local MHit :Int = MouseHit( MOUSE_LEFT )
		Local MHit2:Int = MouseHit( MOUSE_RIGHT )
		Local MX   :Int = MouseX()
		Local MY   :Int = MouseY()
		
		For Local Atom:TAtom = EachIn Atoms
			Atom.Hover = False
			
			If ( MouseX() - Atom.X )^2 + ( MouseY() - Atom.Y )^2 < Atom.Radius^2 Then
				Atom.Hover = True
				
				If MHit Then
					If Atom.Color = 1 Then
						Atom.Selected = True
						
						MHit = False
					Else
						For Local Origin:TAtom = EachIn Atoms
							If Origin.Selected Then Origin.SendElectrons( Atom )
						Next
					EndIf
				EndIf
				
				If MHit2 And Atom.Color = 1 Then
					For Local Origin:TAtom = EachIn Atoms
						If Origin.Selected Then Origin.SendElectrons( Atom )
						
						Origin.Selected = False
					Next
					
					MHit2 = False
				EndIf
			EndIf
		Next
		
		If MHit Then
			For Local Atom:TAtom = EachIn Atoms
				Atom.Selected = False
			Next
		EndIf
	End Function
	
	Function Render()
		Local HoverAtom:TAtom
		
		RenderElectrons( True )
		
		For Local Atom:TAtom = EachIn Atoms
			SetColor 255, 255, 255
			SetRotation Atom.Angle
			DrawImage Frames, Atom.X + 4, Atom.Y + 3, Atom.Charge[ KIND_NEUTRON ]/10*10 + Atom.Charge[ KIND_PROTON ]/10
			
			SetRotation 0.0
			
			If Atom.Hover Or Atom.Selected Then
				SetColor 128 + Atom.Selected*64, 128 + Atom.Selected*64, 128 + Atom.Selected*64
				DrawImage SelectionImg, Atom.X - 30, Atom.Y - 30
				
				If Atom.Hover Then HoverAtom = Atom
			EndIf
			
			If Atom.Color Then
				SetScale 1.0, 1.0
				SetAlpha 0.6
				SetColor Colors[ Atom.Color ][ 0 ], Colors[ Atom.Color ][ 1 ], Colors[ Atom.Color ][ 2 ]
				If Max( Max( Atom.Charge[ 0 ], Atom.Charge[ 1 ] ), Atom.Charge[ 2 ] ) < 10 Then
					DrawImage TextRingNarrow, Atom.X + 30.0, Atom.Y - 27.5
				Else
					DrawImage TextRing, Atom.X + 30.0, Atom.Y - 27.5
				EndIf
				
				SetAlpha 1.0
				SetScale 0.9, 0.9
				For Local T:Int = 0 To 2
					If Atom.Charge[ T ] >= 10 Then
						DrawImage Font, Atom.X + 37, Atom.Y - 22 + T*14, Atom.Charge[ T ]/10
						DrawImage Font, Atom.X + 47, Atom.Y - 22 + T*14, Atom.Charge[ T ] Mod 10
					Else
						DrawImage Font, Atom.X + 37, Atom.Y - 22 + T*14, Atom.Charge[ T ]
					EndIf
				Next
			EndIf
			
			SetScale 1.0, 1.0
		Next
		
		RenderElectrons( False )
		
		Local MX:Int = MouseX()
		Local MY:Int = MouseY()
		
		SetColor 255, 255, 255
		
		For Local Atom:TAtom = EachIn Atoms
			If Atom.Selected And Not Atom.Hover Then
				If HoverAtom Then
					Local DX:Int = HoverAtom.X - Atom.X
					Local DY:Int = HoverAtom.Y - Atom.Y
					
					Local Dist:Float = Sqr( DX*DX + DY*DY )
					
					Local DirX:Float = DX/Dist
					Local DirY:Float = DY/Dist
					
					Local StartX:Float = Atom.X + DirX*Atom.Radius + 3
					Local StartY:Float = Atom.Y + DirY*Atom.Radius + 2
					
					Local EndX:Float = HoverAtom.X - DirX*HoverAtom.Radius + 3
					Local EndY:Float = HoverAtom.Y - DirY*HoverAtom.Radius + 2
					
					DX = EndX - StartX
					DY = EndY - StartY
					
					SetAlpha 0.5
					SetScale Sqr( DX*DX + DY*DY )/16.0, 0.5
					SetRotation ATan2( DY, DX )
					
					DrawImage LineImg, StartX, StartY
					
					SetScale 1.0, 1.0
					SetRotation 0.0
				Else
					Local DX:Int = MX - Atom.X
					Local DY:Int = MY - Atom.Y
					
					Local Dist:Float = Sqr( DX*DX + DY*DY )
					
					Local DirX:Float = DX/Dist
					Local DirY:Float = DY/Dist
					
					Local StartX:Float = Atom.X + DirX*Atom.Radius + 3
					Local StartY:Float = Atom.Y + DirY*Atom.Radius + 2
					
					DX = MX - StartX
					DY = MY - StartY
					
					SetAlpha 0.5
					SetScale Sqr( DX*DX + DY*DY )/16.0, 0.5
					SetRotation ATan2( DY, DX )
					
					DrawImage LineImg, StartX, StartY
					
					SetScale 1.0, 1.0
					SetRotation 0.0
				EndIf
			EndIf
			
			SetAlpha 1.0
		Next
		
		Local OldX:Float = 0, EmptyCount:Int
		For Local Index:Int = 1 Until ParticleCount.Length
			If ParticleCount[ Index ] Then
				SetColor Colors[ Index ][ 0 ], Colors[ Index ][ 1 ], Colors[ Index ][ 2 ]
				DrawRect OldX, GHeight - 10, GWidth*ParticleCount[ Index ]/Float( ParticleCount[ 0 ] ), 10
				
				OldX :+ GWidth*ParticleCount[ Index ]/Float( ParticleCount[ 0 ] )
			ElseIf Index = 1 Then
				SetColor 255, 255, 0
				DrawImage GameOverImage, GWidth/2, GHeight/2
				Flip 0
				
				WaitKey()
				
				GameState = STATE_MENU
			Else
				EmptyCount :+ 1
			EndIf
		Next
		
		If EmptyCount = ParticleCount.Length - 2 Then
			If CurrLevel = Levels.Length - 1 Then
				SetColor 255, 255, 0
				DrawImage YouWinImage, GWidth/2, GHeight/2
				Flip 0
				
				WaitKey()
				
				GameState = STATE_MENU
			Else
				LoadNextLevel()
			EndIf
		EndIf
		
		SetColor 255, 255, 255
	End Function
	
	Function RenderElectrons( Culling:Int )
		SetScale 0.15, 0.15
		SetBlend LIGHTBLEND
		SetAlpha 0.3
		
		For Local Atom:TAtom = EachIn Atoms
			Local RotX:Float, RotY:Float, RotZ:Float
			
			SetColor Colors[ Atom.Color ][ 0 ], Colors[ Atom.Color ][ 1 ], Colors[ Atom.Color ][ 2 ]
			
			For Local I:Int = 0 Until Atom.Charge[ KIND_ELECTRON ]/2
				RotX = Cos( I*36 )*20
				RotY = Sin( I*36 )*20
				
				RotateAroundAxis( RotX, RotY, RotZ, 0.0, 0.0, 1.0, Atom.RingRotation[ 0, I ], RotX, RotY, RotZ )
				RotateAroundAxis( RotX, RotY, RotZ, Sin( I*10 ), Cos( I*10 ), 0.0, Atom.RingRotation[ 1, I ], RotX, RotY, RotZ )
				
				If ( RotZ < 0.0 And Culling ) Or ( RotZ >= 0.0 And Not Culling ) Then ..
					DrawImage TAtom.ElectronImage, Atom.X + RotX, Atom.Y + RotY
			Next
		Next
		
		SetScale 1.0, 1.0
		SetAlpha 1.0
		SetBlend ALPHABLEND
	End Function
	
	Function Update()
		Local Ratio:Float = ( MilliSecs() Mod 3600 )/10.0
		
		For Local I:Int = 0 Until ParticleCount.Length
			ParticleCount[ I ] = 0
		Next
		
		For Local Atom:TAtom = EachIn Atoms
			For Local I:Int = 0 Until 50
				Atom.RingRotation[ 0, I ] = ( Atom.RingRotation[ 0, I ] + 3.0 + ( I + 1 )*0.01 ) Mod 360
				Atom.RingRotation[ 1, I ] = ( Atom.RingRotation[ 1, I ] + 5.0 + ( I + 1 )*0.01 ) Mod 360
			Next
			
			Atom.Angle :+ Atom.TurnSpeed
			
			If Atom.Color Then
				Atom.CurrElectron :+ Production[ Atom.Kind ]*DeltaTime
				
				If Atom.CurrElectron > 1.0 Then
					Atom.Charge[ Atom.Kind ] = Min( Atom.Charge[ Atom.Kind ] + 1, 99 )
					
					Atom.CurrElectron :- 1.0
				EndIf
				
				ParticleCount[ Atom.Color ] :+ Atom.Charge[ 0 ] + Atom.Charge[ 1 ] + Atom.Charge[ 2 ]
			EndIf
		Next
	End Function
End Type

Type TAttackAtom
	Global Speed:Float[] = [ 2.0, 1.0, 0.5 ]
	
	Global Attacks:TList = New TList
	
	Field Color:Int
	
	Field X:Float
	Field Y:Float
	
	Field Charge:Int[ 3 ]
	
	Field Target:TAtom
	
	Field Group:TAttackGroup
	
	Method New()
		Attacks.AddLast( Self )
	End Method
	
	Function Create:TAttackAtom( Origin:TAtom, Target:TAtom, Charge:Int[], Group:TAttackGroup = Null )
		Local AttackAtom:TAttackAtom = New TAttackAtom
		
		AttackAtom.Color = Origin.Color
		AttackAtom.Target = Target
		AttackAtom.Charge = Charge
		
		AttackAtom.X = Origin.X
		AttackAtom.Y = Origin.Y
		
		If Group Then
			Group.Color			= AttackAtom.Color
			Group.Target		= Target
			
			Group.Size			:+ 1
			Group.Charge[ 0 ]		:+ Charge[ 0 ]
			Group.Charge[ 1 ]		:+ Charge[ 1 ]
			Group.Charge[ 2 ]		:+ Charge[ 2 ]
			
			Local DX:Float = AttackAtom.Target.X - AttackAtom.X
			Local DY:Float = AttackAtom.Target.Y - AttackAtom.Y
			
			Local Length:Float = Sqr( DX*DX + DY*DY )
			
			Local SmallestSpeed:Float
			For Local I:Int = 0 To 2
				If Charge[ I ] Then SmallestSpeed = Speed[ I ]
			Next
			
			If Group.LastTime < Length/SmallestSpeed Then
				If Group.LastAttack Then Group.LastAttack.Group = Null
				
				Group.LastTime = Length/SmallestSpeed
				Group.LastAttack = AttackAtom
				AttackAtom.Group = Group
			EndIf
		EndIf
		
		Return AttackAtom
	End Function
	
	Function Render()
		RenderElectrons( True )
		
		For Local Atom:TAttackAtom = EachIn Attacks
			SetColor 255, 255, 255
			SetRotation -Atom.Target.Angle
			DrawImage TAtom.Frames, Atom.X + 4, Atom.Y + 3, Atom.Charge[ KIND_NEUTRON ]/10*10 + Atom.Charge[ KIND_PROTON ]/10
			
			SetRotation 0.0
		Next
		
		RenderElectrons( False )
	End Function
	
	Function RenderElectrons( Culling:Int )
		SetScale 0.15, 0.15
		SetBlend LIGHTBLEND
		SetAlpha 0.3
		
		For Local Atom:TAttackAtom = EachIn Attacks
			Local RotX:Float, RotY:Float, RotZ:Float
			
			SetColor Colors[ Atom.Color ][ 0 ], Colors[ Atom.Color ][ 1 ], Colors[ Atom.Color ][ 2 ]
			
			For Local I:Int = 0 Until Atom.Charge[ KIND_ELECTRON ]/2
				RotX = Cos( I*36 )*20
				RotY = Sin( I*36 )*20
				
				RotateAroundAxis( RotX, RotY, RotZ, 0.0, 0.0, 1.0, Atom.Target.RingRotation[ 0, I ], RotX, RotY, RotZ )
				RotateAroundAxis( RotX, RotY, RotZ, Sin( I*10 ), Cos( I*10 ), 0.0, Atom.Target.RingRotation[ 1, I ], RotX, RotY, RotZ )
				
				If ( RotZ < 0.0 And Culling ) Or ( RotZ >= 0.0 And Not Culling ) Then ..
					DrawImage TAtom.ElectronImage, Atom.X + RotX, Atom.Y + RotY
			Next
		Next
		
		SetScale 1.0, 1.0
		SetAlpha 1.0
		SetBlend ALPHABLEND
	End Function
	
	Function Update()
		For Local AttackAtom:TAttackAtom = EachIn Attacks
			Local DX:Float = AttackAtom.Target.X - AttackAtom.X
			Local DY:Float = AttackAtom.Target.Y - AttackAtom.Y
			
			Local Length:Float = DX*DX + DY*DY
			
			If Length < AttackAtom.Target.Radius*AttackAtom.Target.Radius Then
				AttackAtom.Target.InsertAttack( AttackAtom )
				
				If AttackAtom.Group Then AttackAtom.Group.LastAttack = Null
				
				Attacks.Remove( AttackAtom )
			EndIf
			
			Local SmallestSpeed:Float
			For Local I:Int = 0 To 2
				If AttackAtom.Charge[ I ] Then SmallestSpeed = Speed[ I ]
			Next
			
			Length = Sqr( Length )
			
			AttackAtom.X :+ DX/Length*SmallestSpeed*DeltaTime
			AttackAtom.Y :+ DY/Length*SmallestSpeed*DeltaTime
			
			TAtom.ParticleCount[ AttackAtom.Color ] :+ AttackAtom.Charge[ 0 ] + AttackAtom.Charge[ 1 ] + AttackAtom.Charge[ 2 ]
		Next
		
		For Local I:Int = 1 Until TAtom.ParticleCount.Length
			TAtom.ParticleCount[ 0 ] :+ TAtom.ParticleCount[ I ]
		Next
	End Function
End Type
