
Type TLevel
	
	Const RADIUS = 0
	Const LINE = 1
	Const FLEXIBLE = 0
	Const HARD = 1
	
	Field CurrentMap:String
	Field XPos:Float[]
	Field YPos:Float[]
	Field Mode:Float[]
	Field Force:Float
	Field Direction:Int
	Field Plots:Int
	Field FixDirection:Int
	Field TransitionWidth:Double
	Field imgFloor:TImage
	Field imgFloorHard:TImage
	Field Speed:Float
	
	Field Checkpoints:TList
	Field Target:TTarget
	
	Function Init:TLevel( lCountPlots, lTop, lImgFloor:TImage, lImgFloorHard:TImage, lTransitionWidth )
		
		t:TLevel = New TLevel
		
		t.XPos = t.XPos[ ..lCountPlots ]
		t.YPos = t.YPos[ ..lCountPlots ]
		t.Mode = t.Mode[ ..lCountPlots ]
		t.Plots = lCountPlots
		
		For I:Float = 0 To lCountPlots-1
			t.XPos[I] = I
			t.YPos[I] = lTop-(I/7)
		Next
		
		t.imgFloor = lImgFloor
		t.imgFloorHard = lImgFloorHard
		t.TransitionWidth = lTransitionWidth
		t.Force = 50
		t.Checkpoints = CreateList()
		t.Target = TTarget.Init( 0, 0 )
		
		Return t
		
	End Function
	
	
	
	Method Reset()
		
		
		
	End Method
	
	
	Method LoadMap()
		
		
		
	End Method
	
	
	Method Transform( lOffX, lOffY, lX, lY, lMode = RADIUS, lRadius = 0, lAngle = 0, MH = 0 )
		
		If lY < YPos[lX+lOffX] Then
			Direction = 1
		Else
			Direction = 0
		EndIf
		
		If lMode = RADIUS Then
			
			Local XEntf:Float
			Local YEntf:Float
			Local Entf:Float
			Local Transition
			Local TBegin
			Local TStartL:Double[2]
			Local TStartR:Double[2]
			Local TDiff:Double[2]
			Local StartValue:Double
			
			For I = 0 To Self.Plots-1
				
				XEntf = Abs(XPos[I] - lX - lOffX)
				YEntf = Abs(YPos[I] - lY - lOffY)
				
				Entf = Sqr( XEntf*XEntf + YEntf*YEntf )
				
				If Entf < 5 Then
					Entf = 5
				EndIf
				
				If I-TransitionWidth > 0 And I+TransitionWidth < Plots
					
					If TBegin = 0 And Mode[I] = FLEXIBLE And Mode[I+1] = HARD Then
						TBegin = 1
						TDiff[0] = YPos[I] - YPos[I+TransitionWidth]
						TStartL[0] = YPos[I-1]
					EndIf
					
					If TBegin = 1 Then
						
						TBegin = 2
						For J = 0 To TransitionWidth-1
							YPos[I+J] = TStartL[0]+(Sin((180.0/TransitionWidth)*J+90)-1)*TDiff[0]/2
						Next
						
					EndIf
					
					If TBegin = 2 And Mode[I] = FLEXIBLE And Mode[I-1] = HARD Then
						TBegin = 3
						TDiff[1] = YPos[I] - YPos[I-TransitionWidth]
						TStartL[1] = YPos[I-TransitionWidth]
					EndIf
					
					
					If TBegin = 3 Then
						
						TBegin = 4
						For J = 0 To TransitionWidth-1
							YPos[I-TransitionWidth+J] = TStartL[1]-(Sin((180.0/TransitionWidth)*J+90)-1)*TDiff[1]/2
						Next
						
					EndIf
					
					If TBegin = 4 Then TBegin = 0
				EndIf
				
				
				If Mode[I] = FLEXIBLE
					If Direction = 1 Then
						YPos[I] :+ Self.Force/Entf
					Else
						YPos[I] :- Self.Force/Entf
					EndIf
				EndIf
				
			Next
		EndIf
		
		
		If lMode = LINE Then
			
			Local Beginning:Int = lX - (lRadius*Cos(lAngle))
			Local Ending:Int = lX + (lRadius*Cos(lAngle))
			Local RelBeginning:Int = Beginning - lX
			Local Diff:Float = (2*Tan(lAngle))-(Tan(lAngle))
			Local YP:Float = lY - (lRadius*Sin(lAngle))
			
			If FixDirection = 1 Then
				For I = Beginning To Ending	
					If I > 0 And I < Plots-1
						If YP > YPos[I] Then YPos[I] = YP
					EndIf
					YP :+ Diff
				Next
			Else
				For I = Beginning To Ending	
					If I > 0 And I < Plots-1
						If YP < YPos[I] Then YPos[I] = YP
					EndIf
					YP :+ Diff
				Next
			EndIf
			
		EndIf
		
	End Method
	
	
	Method Draw( lOffX, lOffY )
		
		Local transition = 1
		Local transitionMax = 0
		
		SetColor 255,255,255
		For I = lOffX To lOffX + TMode.GraphX
			If I < Plots
				If transition = TransitionWidth Then transitionMax = 1
				If transition = 1 Then transitionMax = 0
				If Not transitionMax And Mode[I] = HARD Then
					transition :+ 1
				ElseIf I+TransitionWidth < Plots And transition > 1 And Mode[I+TransitionWidth] = FLEXIBLE Then 
					transition :- 1
				EndIf
				If Mode[I] = FLEXIBLE Then
					DrawImage Self.imgFloor, Self.XPos[I]-lOffX, Self.YPos[I]-lOffY
				Else
					DrawImage imgFloorHard, Self.XPos[I]-lOffX, Self.YPos[I]-lOffY, transition
				EndIf
			Else
				Exit
			EndIf
		Next
		
		For CP:TCheckpoint = EachIn Checkpoints
			CP.SX = TScroll.ScrollX
			CP.SY = TScroll.ScrollY
			CP.Draw()
		Next
		
		Target.SX = TScroll.ScrollX
		Target.SY = TScroll.ScrollY
		Target.Draw()
		
	End Method
	
	
	Method SetForce( R )
		
		Force = R
		
	EndMethod
	
	
	Method SetMode( lX, lW, lScrollX, lMode )
		
		If lW < 2*TransitionWidth Then lW = 2*TransitionWidth
		
		For I = lScrollX+lX To lScrollX+lX+lW
			If I<Plots Then Mode[I] = lMode
		Next
		
	End Method
	
	
	Method AddCheckpoint( lC:TCheckpoint )
		
		Checkpoints.AddLast( lC )
		
	End Method
	
	
	Method SetTarget( lX, lY )
		
		Target.X = lX
		Target.Y = lY
		
	End Method
	
	
	Method UpdateCheckpoints( lX, lY, lR )
		
		Local all = 1
		For C:TCheckpoint = EachIn Checkpoints
			C.Hit( lX, lY, lR )
			If Not C.State Then all = 0
		Next
		
		If all Then Target.Hit( lX, lY, lR )
		
		Return Target.State
		
	End Method
	
End Type





