
Type TPhysics
	
	Field GravityX:Float
	Field GravityY:Float
	Field AxisX:Float
	Field AxisY:Float
	Field Dist:Float
	Field Player:TPlayer
	
	Const TIMESTEP:Int = 1
	
	Function Init:TPhysics( lgravX, lgravY, lPlayer:TPlayer )
		
		t:TPhysics = New TPhysics
		
		t.GravityX = lgravX
		t.GravityY = lgravY
		t.Player = lPlayer
		
		Return t
		
	End Function
	
	
	Method Update( X1:Float, Y1:Float, X2:Float, Y2:Float )
		
		UpdateForce()
		UpdatePlayer()
		
		AxisX = -( Y2 - Y1 )
		AxisY =  ( X2 - X1 )
		
		Normalize( AxisX, AxisY )
		
		Dist = CalcMinDist( X1, Y1, Player.R*AxisX+Player.X, Player.R*AxisY+Player.Y, AxisX, AxisY )
		
		Player.X :- Dist*AxisX
		Player.Y :- Dist*AxisY
		
	End Method
	
	
	Method UpdateForce()
		
		Player.AX = GravityX
		Player.AY = GravityY
		
	End Method
	
	
	Method Normalize( X:Float Var, Y:Float Var )
		
		Local length:Float = Sqr( X*X + Y*Y )
		X = 1/length * X
		Y = 1/length * Y
		
	End Method
	
	
	Method CalcMinDist:Float( A1X:Float, A1Y:Float, QX:Float, QY:Float, AxisX:Float, AxisY:Float )
		
		Local projectQ:Float = QX*AxisX + QY*AxisY
		Local projectA:Float = A1X*AxisX + A1Y*AxisY
		
		If projectA-projectQ < 0 Then Return (Abs( projectA-projectQ ))
		
		Return 0
		
	End Method
	
	
	Method UpdatePlayer()
		
		Local TempOldX:Float = Player.X
		Local TempOldY:Float = Player.Y
		
		Player.X = 2*Player.X - Player.OldX + Player.AX*TIMESTEP*TIMESTEP*0.5
		Player.Y = 2*Player.Y - Player.OldY + Player.AY*TIMESTEP*TIMESTEP*0.5
		
		Player.OldX = TempOldX
		Player.OldY = TempOldY
		
	End Method
	
	
	Method SetForces( X:Float, Y:Float )
		
		GravityX = X
		GravityY = Y
		
	End Method
	
End Type